import datetime
import os

import numpy as np
import pandas as pd
from cached_property import cached_property

from gerby.clients import ForecastClient
from gerby.constants import HOURS_PER_DAY, SECONDS_PER_DAY


class Assignments:
    def __init__(self, start_date=None, end_date=None):
        self.__f = ForecastClient()
        (self.start, self.end) = self.__define_date_range(start_date, end_date)

    @cached_property
    def all(self):
        return self.get_assignments()

    def __define_date_range(self, start_date, end_date):
        if start_date is not None and end_date is not None:
            return (start_date, end_date)

        today = datetime.date.today()
        sunday = today - datetime.timedelta(days=(today.weekday() + 1))
        saturday = sunday + datetime.timedelta(days=6)
        return (sunday, saturday)

    def annotate(self, assignments_df):
        df = assignments_df.copy()
        df["start_date"] = pd.to_datetime(df["start_date"])
        df["allocation"] = df["allocation"].fillna(SECONDS_PER_DAY)
        df = df.assign(
            n_days=lambda x: (
                (pd.to_datetime(x["end_date"]) - pd.to_datetime(x["start_date"]))
                / np.timedelta64(1, "D")
                + 1
            ),
            hours_per_day=lambda x: x["allocation"] / 60 / 60,
        )
        df["work_week"] = df["start_date"] - pd.to_timedelta(
            df["start_date"].dt.dayofweek, unit="d"
        )

        df = df.reset_index(drop=True)
        # round up hours per day to full day
        df = df.assign(
            hours_per_day=np.where(
                df["hours_per_day"] > HOURS_PER_DAY * 0.75,
                HOURS_PER_DAY,
                df["hours_per_day"],
            )
        )
        df["forecasted_hours"] = df["n_days"] * df["hours_per_day"]
        return df

    def get_assignments(self):
        assignments = self.__f.assignments(self.start.isoformat(), self.end.isoformat())
        df = pd.DataFrame(assignments["assignments"])
        df = df.rename(
            columns={
                "person_id": "forecast_user_id",
                "project_id": "forecast_project_id",
            }
        )
        return self.annotate(df)

    def grouped_by_week(self):
        df = self.all
        df = pd.DataFrame(
            df.groupby(["work_week", "forecast_user_id", "forecast_project_id"]).sum()[
                "forecasted_hours"
            ]
        ).reset_index()
        return df

    def grouped(self):
        df = self.all
        df = pd.DataFrame(
            df.groupby(["forecast_user_id", "forecast_project_id"]).sum()[
                "forecasted_hours"
            ]
        ).reset_index()
        return df



import os

import requests

from gerby.secrets import GLASSFROG_API_TOKEN


class GlassFrogClient:
    """GlassFrogClient for interacting with GlassFrog api"""

    base_url = "https://api.glassfrog.com/api/v3"

    def __init__(self, token=GLASSFROG_API_TOKEN):
        """
        Args:
            token (str): GlassFrog api token
        Returns:
            instance of GlassFrog client
        """
        self.headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
            "X-Auth-Token": token,
        }

    def assignments(self):
        return self.__get("/assignments")

    def people(self):
        return self.__get("/people")

    def roles(self):
        return self.__get("/roles")

    def circles(self):
        return self.__get("/circles")

    def policies(self):
        return self.__get("/policies")

    def __parse_response(self, r):
        if r.status_code >= 400:
            return {"error": r.json()}
        else:
            return r.json()

    def __get(self, url):
        return self.__parse_response(
            requests.get(self.base_url + url, headers=self.headers)
        )



import os

import numpy as np
import pandas as pd
from cached_property import cached_property

from gerby.clients import ForecastClient, HarvestClient, SlackClient


class Users:
    def __init__(self):
        self.__h = HarvestClient()
        self.__f = ForecastClient()
        self.__s = SlackClient()

    @cached_property
    def all(self):
        return self.aggregate()

    def __get_email_from_slack_user(self, row):
        if "email" in row["profile"].keys():
            return row["profile"]["email"]
        else:
            return None

    def get_all_slack_users(self):
        # get all the users
        users_info = self.__s.api_call("users.list")
        # cram into dataframe
        df = pd.DataFrame(users_info["members"])
        # keep only those we care about
        team_members = df[
            np.logical_and(df["is_restricted"] == False, df["is_bot"] == False)
        ]
        # pull the email from the profile
        team_members = team_members.assign(
            email=team_members.apply(self.__get_email_from_slack_user, axis=1)
        )[["real_name", "email", "id", "name"]]
        # keep those that have emails (bye bye slackbot)
        team_members = team_members[pd.isnull(team_members["email"]) == False]
        # rename id to slack id
        team_members = team_members.rename(
            columns={"id": "slack_user_id", "name": "slack_handle"}
        )
        return team_members

    def get_all_forecast_users(self):
        df = pd.DataFrame(self.__f.people)
        df = df[df["archived"] != True][["email", "id", "harvest_user_id"]]
        df = df.rename(columns={"id": "forecast_user_id"})[
            ["email", "forecast_user_id", "harvest_user_id"]
        ]
        return df

    def get_all_harvest_users(self):
        # fetch all from harvest
        df = pd.DataFrame(self.__h.fetch_all("users"))
        # rename our columns appropriately
        df = df.rename(columns={"id": "harvest_user_id"})
        df = df[df["is_active"]]
        df = df.assign(billable=df["roles"].apply(lambda x: "Billable" in x))
        df = df.assign(engineer=df["roles"].apply(lambda x: "Engineer" in x))
        df = df.assign(designer=df["roles"].apply(lambda x: "UI/UX design" in x))
        df = df.assign(pm=df["roles"].apply(lambda x: "Product Manager" in x))
        df = df.assign(check_daily=np.logical_or(df["engineer"], df["designer"]))
        df = df.assign(check_weekly=df["billable"])
        return df

    def aggregate(self):
        slack_users = self.get_all_slack_users()
        harvest_users = self.get_all_harvest_users()
        forecast_users = self.get_all_forecast_users()
        return slack_users.merge(harvest_users, how="left").merge(
            forecast_users, how="left"
        )



import base64
import os

import pandas as pd
import requests

from gerby.secrets import HARVEST_ACCOUNT_ID, HARVEST_TOKEN


class HarvestClient(object):
    """HarvestClient for interacting with harvest api v2.0"""

    base_url = "https://api.harvestapp.com/v2/"
    time_off_id = 16121361

    def __init__(self, token=HARVEST_TOKEN, account_id=HARVEST_ACCOUNT_ID):
        """
        Args:
            token (str): harvest api token
            account_id (int): harvest account id
        Returns:
            instance of harvest client
        """
        self.headers = {
            "accept": "application/json",
            "authorization": "Bearer " + token,
            "Harvest-Account-ID": account_id,
            "Content-Type": "application/json",
        }

    def _post(self, url, **kwargs):
        response = requests.post(
            self.base_url + url, headers=self.headers, params=kwargs
        )
        response.raise_for_status()
        return response.json()

    def _get(self, url, **kwargs):
        response = requests.get(
            self.base_url + url, headers=self.headers, params=kwargs
        )
        response.raise_for_status()
        return response.json()

    def users(self, **kwargs):
        """Invoices
        Kwargs:
            is_active
            updated_since
            page
            per_page
        """
        return self._get("users", **kwargs)

    def invoice(self, invoice_id):
        """Invoice
        Kwargs:
            invoice_id (required)
        """
        return self._get("invoices/" + str(invoice_id))

    def invoices(self, **kwargs):
        """Invoices
        Kwargs:
            is_active
            client_id
            updated_since
            page
            per_page
        """
        return self._get("invoices", **kwargs)

    def projects(self, **kwargs):
        """Projects
        Kwargs:
            is_active
            client_id
            updated_since
            page
            per_page
        """
        return self._get("projects", **kwargs)

    def clients(self, **kwargs):
        """Clients
        Kwargs:
            is_active
            updated_since
            page
            per_page
        """
        return self._get("clients", **kwargs)

    def user_project_assignments(self, project_id, **kwargs):
        """User Project Assignments
        Args:
            project_id
        Kwargs:
            updated_since
            page
            per_page
        """
        url = "projects/" + str(int(project_id)) + "/user_assignments"
        return self._get(url, **kwargs)

    def create_user_project_assignments(self, project_id, **kwargs):
        """User Project Assignments
        Args:
            project_id
        Kwargs:
            user_id
            is_active
            is_project_manager
            hourly_rate
            budget
        """
        url = "projects/" + str(int(project_id)) + "/user_assignments"
        return self._post(url, **kwargs)

    def time_entries(self, **kwargs):
        """Time Entries
        Kwargs:
            user_id
            client_id
            is_billed
            is_running
            updated_since
            from
            to
            page
            per_page
        """
        url = "time_entries"
        return self._get(url, **kwargs)

    def fetch_all(self, attribute_type, **kwargs):
        """Fetch All
        Returns the full collection of objects from the harvest api.
        Wraps each of the other methods on HarvestClient
        Args:
            attribute_type - harvest client method name, ex: clients -or- time_entries
        Kwargs:
            (any) - these are simply passed down to the method specified
        """

        ll = []
        page = 1
        while True:
            result = getattr(self, attribute_type)(page=page, **kwargs)
            page = page + 1
            ll.extend(result[attribute_type])
            if not result["next_page"]:
                break
        return ll



from .forecast_client import ForecastClient
from .glass_frog_client import GlassFrogClient
from .guru_client import GuruClient
from .harvest_client import HarvestClient
from .slack_client import SlackClient



import os

import numpy as np
import pandas as pd
from cached_property import cached_property

from gerby.clients import ForecastClient, HarvestClient

from .helpers import pull_model_attributes


class Projects:
    def __init__(self):
        self.__h = HarvestClient()
        self.__f = ForecastClient()

    @cached_property
    def all(self):
        return self.aggregate()

    def get_all_harvest_projects(self):
        df = pd.DataFrame(self.__h.fetch_all("projects")).rename(
            columns={"id": "harvest_project_id"}
        )
        df = pull_model_attributes(df, {"client": ["id", "name"]})
        df = df[["client_id", "client_name", "name", "harvest_project_id", "is_active"]]
        return df

    def get_all_forecast_projects(self):
        df = pd.DataFrame(self.__f.projects)
        df = df.rename(
            columns={"id": "forecast_project_id", "harvest_id": "harvest_project_id"}
        )[["forecast_project_id", "name", "code", "harvest_project_id"]]
        return df

    def merge_and_sync_time_off(self, h, f):
        projects = h.merge(f, how="outer")

        projects["harvest_project_id"] = np.where(
            # conditional
            projects["forecast_project_id"] == self.__f.time_off_id,
            # if true
            self.__h.time_off_id,
            # else
            projects["harvest_project_id"],
        )

        projects["forecast_project_id"] = np.where(
            # conditional
            projects["harvest_project_id"] == self.__h.time_off_id,
            # if true
            self.__f.time_off_id,
            # else
            projects["forecast_project_id"],
        )

        dup = projects[
            np.logical_and(
                projects.forecast_project_id == self.__f.time_off_id,
                pd.isna(projects.client_id),
            )
        ].index

        projects = projects.drop(dup)

        return projects

    def aggregate(self):
        h = self.get_all_harvest_projects()
        f = self.get_all_forecast_projects()
        merged = self.merge_and_sync_time_off(h, f)
        return merged



import os

import requests as r
from cached_property import cached_property

import gerby.constants as c
from gerby.secrets import GURU_PASS, GURU_USER

POLICY_BOARD = "29611c28-41cc-4b25-86fe-30cc7e9d0630"


class HoloCard:

    gf_mirror_collection_id = "fc4e13bf-fa63-4a74-b957-738c9cc85387"

    def __init__(self, title, content, board_id=POLICY_BOARD):
        self.card_data = {
            "preferredPhrase": title,
            "content": content,
            "verificationInterval": 90,
            "shareStatus": "TEAM",
            "boards": [{"id": board_id}],
            "collection": {"id": self.gf_mirror_collection_id},
            "verifiers": [],
        }

    def __repr__(self):
        return f"<HoloCard '{self.card_data['preferredPhrase']}' >"

    @cached_property
    def data(self):
        return self.card_data


class GuruClient:
    __base_url = "https://api.getguru.com/api/v1/"

    def __init__(self):
        self.__user = GURU_USER
        self.__pass = GURU_PASS

    def __build_url(self, path):
        return self.__base_url + path

    def get(self, path):
        full_url = self.__build_url(path)
        return r.get(full_url, auth=(self.__user, self.__pass))

    def post(self, path, data):
        full_url = self.__build_url(path)
        return r.post(full_url, json=data, auth=(self.__user, self.__pass))

    def delete(self, path):
        full_url = self.__build_url(path)
        return r.delete(full_url, auth=(self.__user, self.__pass))

    def query(self, query_string):
        query = urlencode({"q": query_string})
        path = f"search/query?{query}"
        return self.get(path)

    def collections(self):
        path = f"collections"
        return self.get(path)

    def boards(self):
        path = f"boards"
        return self.get(path)

    def board(self, board_id):
        path = f"boards/{board_id}"
        return self.get(path)

    def create_card(self, card_data):
        path = f"cards/extended"
        return self.post(path, card_data)

    def create_group(self, group_name):
        path = f"groups"
        return self.post(path, {"name": group_name})

    def groups(self):
        path = f"groups"
        return self.get(path)

    def members(self):
        path = f"members"
        return self.get(path)

    def delete_card(self, card_id):
        path = f"cards/{card_id}"
        return self.delete(path)

    def clean_board(self, board_id, dry_run=True):
        """
        WARNING: THIS IS A DESTRUCTIVE ACTION, REQUIRES YOU TO OVERRIDE DRY_RUN
        PARAMETER TO ACTUALLY RUN DELETE METHOD.
        """
        board = self.board(board_id).json()
        cards = board["items"]
        print(
            f"cleaning board: {board['title']}, {len(cards)} items to delete [DRY_RUN: ${dry_run}]"
        )
        for (index, title) in [
            (card["id"], card.get("preferredPhrase", "empty-attribute"))
            for card in cards
        ]:
            print(f"deleting: {index}-{title}")
            if not dry_run:
                self.delete_card(index)



import datetime
import os

import numpy as np
import pandas as pd
from cached_property import cached_property

from gerby.clients import HarvestClient

from .helpers import pull_model_attributes


class Entries:
    def __init__(self, start_date=None, end_date=None):
        self.__h = HarvestClient()

        (self.start, self.end) = self.__define_date_range(start_date, end_date)

    @cached_property
    def all(self):
        return self.aggregate()

    def __define_date_range(self, start_date, end_date):
        if start_date is not None and end_date is not None:
            return (start_date, end_date)

        today = datetime.date.today()
        monday = today - datetime.timedelta(days=today.weekday())
        return (monday, today)

    def aggregate(self):
        entries = pd.DataFrame(
            self.__h.fetch_all("time_entries", **{"from": self.start, "to": self.end})
        )

        entries = pull_model_attributes(
            entries,
            {
                "user": ["id", "name"],
                "project": ["id", "name"],
                "client": ["id", "name"],
            },
        )
        entries = entries.drop(columns=["user", "project", "client"])
        entries = entries.rename(
            columns={
                "project_id": "harvest_project_id",
                "user_id": "harvest_user_id",
                "billable": "hours_are_billable",
            }
        )
        entries.index = entries.spent_date
        return entries

    def grouped(self):
        grouped = self.all.groupby(
            [
                "harvest_user_id",
                "hours_are_billable",
                "client_name",
                "client_id",
                "project_name",
                "harvest_project_id",
            ]
        ).sum()["hours"]
        return pd.DataFrame(grouped).reset_index()



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['gerby', 'gerby.clients', 'gerby.data']

package_data = \
{'': ['*']}

install_requires = \
['cached_property>=1.5,<2.0',
 'numpy>=1.17,<2.0',
 'pandas>=0.25.0,<0.26.0',
 'requests>=2.22,<3.0',
 'slackclient==1.3.2']

setup_kwargs = {
    'name': 'gerby',
    'version': '0.1.1',
    'description': 'Core functionality for Very data analysis.',
    'long_description': None,
    'author': 'Jeff McGehee',
    'author_email': 'jeff@verypossible.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import os

import requests
from cached_property import cached_property

from gerby.secrets import FORECAST_ACCOUNT_ID, FORECAST_TOKEN


class ForecastClient:

    base_url = "https://api.forecastapp.com/"
    time_off_id = 3241

    def __init__(self, account_id=FORECAST_ACCOUNT_ID, token=FORECAST_TOKEN):

        self.headers = {
            "accept": "application/json",
            "authorization": "Bearer " + token,
            "Forecast-Account-ID": account_id,
            "Content-Type": "application/json",
        }

        if "error" in self.whoami():
            raise Exception("Forecast not authed")

    @cached_property
    def projects(self):
        return self.get_projects()["projects"]

    @cached_property
    def clients(self):
        return self.get_clients()["clients"]

    @cached_property
    def people(self):
        return self.get_people()["people"]

    def __parse_response(self, r):
        if r.status_code > 400:
            return {"error": r.json()}
        else:
            return r.json()

    def __get(self, url):
        return self.__parse_response(
            requests.get(self.base_url + url, headers=self.headers)
        )

    def whoami(self):
        return self.__get("whoami")

    def get_projects(self):
        return self.__get("projects")

    def get_clients(self):
        return self.__get("clients")

    def get_people(self):
        return self.__get("people")

    def assignments(self, start_date, end_date):
        url = f"assignments?end_date={end_date}&start_date={start_date}&state=active"
        return self.__get(url)




def pull_model_attributes(df_in, attributes):
    df = df_in.copy()
    for (model_name, attributes) in attributes.items():
        for attr in attributes:
            df[model_name+'_'+attr] = df[model_name].apply(lambda x: x[attr])
    return df
